###########################################################################################################
#
#  blackbox
#  
#  @author simon jokuschies
#  @version 3.0
#  @contact info@leafpictures.de
#  @website www.leafpictures.de
#
#  description:
#  automatically create backups of your script whenever you save it.
#  you can choose between a global backup directory or a backup directory per script directly sitting next
#  to your nuke script.
#  works with normal save, incremental save and save as.
#
#  instalation
#
#  put the whole blackbox folder in nuke home directory
#  in init.py write (without the '#'):
#  
#  nuke.pluginAddPath("blackbox")
#
#  Addition for version 2.0
#  set backupMinute, default set to true (description see below line 50)
#
#  Addition for version 3.0
#  Menue entry for showing all backups of the script. When you save your script a screenshot is also saved besides
#  your script to better visualize which stage of work you where when you saved your script. This results in faster
#  locatin the backup you need. 
#
##########################################################################################################

#no need to change anything from here. Edit only if you exactly know what you're doing.

import nuke
import nukescripts
import os
import time
import getpass
import blackboxHelper
import sys
import thread
from PySide.QtGui import *
from PySide.QtCore import *

global backupDir
global enableBackup
global backupPath
global numberOfBackups
global backupSettingsPath
global backupSettings
global backupMinute

backupSettingsPath = "/Users/%s/.nuke/blackbox" % getpass.getuser()
backupSettings = backupSettingsPath+"/blackbox.set"

# In a nutshell you won't have more than one backup per minute and thus not having 3 million backups of your nuke script.
# save backup only for every minute and not for every second.
# If set to true the name of the backup path contains only date_Hour-Minute. So you cannot produce a backup for every second.
# This results in not having 3 Million copies if you always press the save button.
# If set to true backups saved within a minute get overwritten.
backupMinute = True


'''
init backup settings values; create blank file with defaults if blackbox.set doesn't exist
'''
if not os.path.isfile(backupSettings):
	if not os.path.isdir(backupSettingsPath):
		os.makedirs(backupSettingsPath)
	loadBackupSettings = open(backupSettings,'w')
	open(backupSettings,"w").write("@enableBackup=1.0\n@backupPath=\n@operation=1.0\n@numberOfBackups=10")
enableBackup=blackboxHelper.getBackupSettings("@enableBackup", backupSettings)
backupPath=blackboxHelper.getBackupSettings("@backupPath", backupSettings)
operation=blackboxHelper.getBackupSettings("@operation", backupSettings)
numberOfBackups=blackboxHelper.getBackupSettings("@numberOfBackups", backupSettings)

class blackboxSettingsPanel(nukescripts.PythonPanel):
	'''
	blackbox panel
	'''
	def __init__( self ):
		'''
		init values
		'''
		enableBackup=blackboxHelper.getBackupSettings("@enableBackup", backupSettings)
		backupPath=blackboxHelper.getBackupSettings("@backupPath", backupSettings)
		operation=blackboxHelper.getBackupSettings("@operation", backupSettings)
		numberOfBackups=int(blackboxHelper.getBackupSettings("@numberOfBackups", backupSettings))
		
		if enableBackup == "1.0":
			enableBackup=True
		else:
			enableBackup=False

		nukescripts.PythonPanel.__init__(self, "Blackbox Settings", "Blackbox Settings")
		self.setMinimumSize(450,160)
	
		self.enableBackup = nuke.Boolean_Knob("enableBackup","enable backup",enableBackup)
		self.backupDirOp=nuke.Enumeration_Knob("operation","operation",["backup directory on global place","backup directory per script"])
		self.backupDirOp.setValue(str(operation))
		self.backupDirOp.resize(300, 30)
		self.backupPath = nuke.File_Knob('backupPath', 'backupPath')
		self.backupPath.setValue(backupPath)
		
		if operation == "0.0":
			self.backupPath.setVisible(True)
		else:
			self.backupPath.setVisible(False)
		
		self.numberOfBackups = nuke.String_Knob('numberOfBackups', 'number of backups')
		self.numberOfBackups.setValue(str(numberOfBackups))
		
		self.addKnob(self.enableBackup)
		self.addKnob(self.backupDirOp)
		self.addKnob(self.backupPath)
		self.addKnob(self.numberOfBackups)
	
	def show( self ):
		'''
		action performed when pressed ok
		'''
		result = nukescripts.PythonPanel.showModalDialog(self)
		
		if result:    
			def editBackupSettings():
				'''
				edit backup settings - write new values to blackbox.set
				'''
				enableBackup = self.enableBackup.getValue()
				backupPath = self.backupPath.getValue()
				backupDirOp = self.backupDirOp.getValue()
				numberOfBackups = self.numberOfBackups.getValue()
			 
				if numberOfBackups.isdigit():   
					if backupDirOp == 0:
						if os.path.exists(backupPath):
							pass
						else:
							nuke.message("The following directory you entered does not exist or is not valid: \n\n %s" % backupPath)
							backupSetter()

					#update settings
					open(backupSettings,"w").write("@enableBackup={0}\n@backupPath={1}\n@operation={2}\n@numberOfBackups={3}".format(str(enableBackup),backupPath,backupDirOp,numberOfBackups))
					toggleBackupProcess()
				else:
					nuke.message("Your input for was wrong. Please enter a number")
					backupSetter()
				
			editBackupSettings()
	
	def knobChanged( self, knob ): 
		'''
		panel knob changed actions
		'''
		enableBackup = self.enableBackup.getValue()
		backupPath = self.backupPath.getValue()
		numberOfBackups = self.numberOfBackups.getValue()
		
		if knob.name() == "operation":
			if self.backupDirOp.getValue() == 0:
				self.backupPath.setVisible(True)
			else:
				self.backupPath.setVisible(False)
		  
def searchBackupFolder(ctrl=False):
	'''
	locate backup folder
	'''

	backupPath=blackboxHelper.getBackupSettings("@backupPath", backupSettings)
	backupOperation=blackboxHelper.getBackupSettings("@operation", backupSettings)
	backupsFolder=""
	backupsFolderGlobalPath=""
	backupsFolderNextToScript=""
	script = nuke.root().name()
	scriptName = (nuke.root().name().split("/")[-1]).replace(".nk","")

	if ctrl:
		print "backupPath: %s" % backupPath
		print "backupOperation: %s" % backupOperation


	#load backups
	#try to read find the backup dirs (backups of current nuke script in global backup directory and next to script directory)
	

	#check if backup folder exists inside global backups dir
	gl="{0}{1}".format(backupPath,scriptName)
	if os.path.isdir(gl):
		if ctrl:
			print "found backup directory in global backup dir (%s)" % gl
	else:
		if ctrl:
			print "no backup directory in global backup dir (%s) found" % gl   
		else:
			pass        
	
	#check if backup folder is located next to current script
	ns="{root}/{backupFolder}{scriptName}".format(root=os.path.dirname(nuke.root().name()),backupFolder="_backups_",scriptName=scriptName)
	if os.path.isdir(ns):
		if ctrl:
			print "found backup directory next to script (%s)" % ns
	else:
		if ctrl:
			print "no backup directory next to script (%s) found" % ns
		else:
			pass

	#global dir
	if backupOperation=="0.0":
		if ctrl:
			print "global backup dir"

		if os.path.isdir(gl):
			backupsFolder=gl
			if ctrl:
				print "set to global Folder"
		else:
			if os.path.isdir(ns):
				backupsFolder=ns
				if ctrl:
					print "set to local Folder"
			else:
				pass
		
	#next to script
	if backupOperation=="1.0":
		if ctrl:
			print "local backup dir"

		if os.path.isdir(ns):
			backupsFolder=ns
			if ctrl:
				print "set to local Folder"
		else:
			if os.path.isdir(ns):
				backupsFolder=gl
				if ctrl:
					print "set to global Folder"
			else:
				pass
	
	if ctrl:
		print "backupFolder: %s" % backupsFolder

	return backupsFolder

class blackboxBackupsPanel(nukescripts.PythonPanel):
	'''
	panel showing all backups of current script
	'''

	def __init__( self ):
		'''
		init values
		'''
		backups=[]
		backupsFolder = searchBackupFolder()

		global backupFileBtn
		global backupFilePreview
		global backupFileDelete

		backupFileBtn={}
		backupFilePreview={}
		backupFileDelete={}

		nukescripts.PythonPanel.__init__(self, "Blackbox script backups ({backupsFolder})".format(backupsFolder=backupsFolder), "Blackbox script backups")
		self.setMinimumSize(850,160)

		for file in os.listdir(backupsFolder):
			fileName, fileExt = os.path.splitext(file)
			if fileExt==".nk":
				backups.append(fileName)
	
		for backupFile in backups:
			self.backupFileBtn = nuke.PyScript_Knob("load_%s" % backupFile, backupFile)
			self.backupFileBtn.setFlag(nuke.STARTLINE)
			self.backupFilePreview = nuke.PyScript_Knob("preview_%s" % backupFile, "preview")
			self.backupFileDelete = nuke.PyScript_Knob("delete_%s" % backupFile, "delete")

			backupFileBtn[backupFile]=self.backupFileBtn
			backupFilePreview[backupFile]=self.backupFilePreview
			backupFileDelete[backupFile]=self.backupFileDelete

			self.addKnob(self.backupFileBtn)
			self.addKnob(self.backupFilePreview)
			self.addKnob(self.backupFileDelete)
	
	def show( self ):
		'''
		action performed when pressed ok
		'''
		result = nukescripts.PythonPanel.showModalDialog(self)
		
		if result:   
			def editBackupSettings():
				pass
	
	def knobChanged( self, knob ): 
		'''
		panel knob changed actions
		'''

		backupsFolder = searchBackupFolder()
		
		if "load" in knob.name():
			
			actFile = backupsFolder + "/" + knob.name().split("load_")[1] + ".nk"

			def openScript():
				nuke.scriptOpen(actFile) 
				return True

			def thread_dispatch():
				return nuke.executeInMainThreadWithResult(openScript, ())
		
			thread.start_new_thread(thread_dispatch, ())

		if "preview" in knob.name():
			actFile = backupsFolder + "/" + knob.name().split("preview_")[1] + ".png"
			img="<img src='{actFile}' />".format(actFile=actFile)
			nuke.message(img)
			
		if "delete" in knob.name():
			actFiles=[]
			actFiles.append(backupsFolder + "/" + knob.name().split("delete_")[1] + ".nk")
			actFiles.append(backupsFolder + "/" + knob.name().split("delete_")[1] + ".png")

			if nuke.ask("Do you really want to delete the backup file?"):
				
				toDelete=knob.name().replace("delete_","")
				
				self.removeKnob(backupFileBtn[toDelete])
				self.removeKnob(backupFilePreview[toDelete])
				self.removeKnob(backupFileDelete[toDelete])

				for f in actFiles:	
					try:
						os.remove(f)
					except:
						nuke.message("erorr deleting{f}".format(f=f))
			else:
				pass
		
def grabScreen(loc):  
   
	'''
	make screenshot and save it besides backupFile
	'''   
	width=1000

	try:
		app = QtGui.QApplication(sys.argv)
	except:
		pass
		#app = QtCore.QCoreApplication.instance()
	try:
		#grab
		QPixmap.grabWindow(QApplication.desktop().winId(),0,0,
		QApplication.desktop().screenGeometry().width()*0.8,
		QApplication.desktop().screenGeometry().height()*0.8).save(loc)
		#resize
		pixmap_resized = QPixmap(loc).scaled(width, width/1.777, Qt.KeepAspectRatio)
		pixmap_resized.save(loc)
	except:
		nuke.message("couldn't write screen content")

	
def deleteOlderBackupVersions(path):
	'''
	delete all the older versions which are older than {all backups - max. backup number}
	'''
	numberOfBackups=int(blackboxHelper.getBackupSettings("@numberOfBackups", backupSettings))

	i=0
	files=""
	for file in os.listdir(path):
		if file.endswith(".nk"):
		   files+="%s," %file
		   i+=1

	filesArr = files.split(",")
	#little hack, delete last comma
	filesArr.pop()
	
	#only keep the latest versions
	if len(filesArr)>numberOfBackups:
		toDelete = len(filesArr) - int(numberOfBackups)
		for f in range(0,toDelete):
			os.remove(path+"/"+filesArr[f])
	
def makeBackup():
	'''
	make backup of script
	'''
	#get script name and make folder if not exist
	script = nuke.root().name()
	scriptName = (nuke.root().name().split("/")[-1]).replace(".nk","")
	operation=blackboxHelper.getBackupSettings("@operation", backupSettings)
	backupPath=blackboxHelper.getBackupSettings("@backupPath", backupSettings)
	numberOfBackups=int(blackboxHelper.getBackupSettings("@numberOfBackups", backupSettings))

	if backupMinute == True:
		t = time.strftime("%y%m%d-%H%M")
	else:
		t = time.strftime("%y%m%d-%H%M%S")

	# global dir
	if operation=="0.0":
		
		if not os.path.isdir(backupPath+"/"+scriptName):
			os.makedirs(backupPath+"/"+scriptName) 
		try:
			nuke.removeOnScriptSave(makeBackup)
			nuke.scriptSave(backupPath+"/"+scriptName+"/bckp_"+t+"_"+scriptName+".nk")
			grabScreen(backupPath+"/"+scriptName+"/bckp_"+t+"_"+scriptName+".png")
			nuke.addOnScriptSave(makeBackup)
		except Exception, e:
			nuke.message("couldn't write a backup file")

		deleteOlderBackupVersions(backupPath+"/"+scriptName)
	# per script
	else:
		backupPath = "/".join(nuke.root().name().split("/")[:-1])+"/_backups_%s" % scriptName
		if not os.path.isdir(backupPath):
			os.makedirs(backupPath)
		try:
			nuke.removeOnScriptSave(makeBackup)
			nuke.scriptSave(backupPath+"/bckp_"+t+"_"+scriptName+".nk")
			grabScreen(backupPath+"/bckp_"+t+"_"+scriptName+".png")
			nuke.addOnScriptSave(makeBackup)
		except Exception, e:
			nuke.message("couldn't write a backup file")

		deleteOlderBackupVersions(backupPath)

def toggleBackupProcess():
	'''
	enable/disable blackbox - set by value in blackbox.set 
	'''
	#reload settings
	enableBackup=blackboxHelper.getBackupSettings("@enableBackup", backupSettings)
	backupPath=blackboxHelper.getBackupSettings("@backupPath", backupSettings)
	operation=blackboxHelper.getBackupSettings("@operation", backupSettings)
	numberOfBackups=blackboxHelper.getBackupSettings("@numberOfBackups", backupSettings)

	if enableBackup == "1.0":
		nuke.addOnScriptSave(makeBackup)
	else:
		nuke.removeOnScriptSave(makeBackup) 

def openBackupDir():
	'''
	open backup directory in explorer
	'''
	backupPath=blackboxHelper.getBackupSettings("@backupPath", backupSettings)
	operation=blackboxHelper.getBackupSettings("@operation", backupSettings)
	scriptName = (nuke.root().name().split("/")[-1]).replace(".nk","")

	# global dir
	if operation=="0.0":
		try:
			blackboxHelper.openFolder(backupPath+"/"+scriptName)
		except Exception, e:
			nuke.message("couldn't open backupPath. no such directory")
	# per script
	else:
		backupPath = "/".join(nuke.root().name().split("/")[:-1])+"/_backups_%s" % scriptName 
		try:
			blackboxHelper.openFolder(backupPath)
		except Exception, e:
			nuke.message("couldn't open backupPath. no such directory")

def backupSetter():
	blackboxSettingsPanel().show()

def showBackupsPanel():
	
	backupsFolder = searchBackupFolder()
	if backupsFolder!="":
		blackboxBackupsPanel().show()
	else:
		nuke.message("Could not find the backup directory for this script. Please make sure that there does exist a backup directory for this current script. Either next to your current script or in your global backup directory. Please refer to the help docs to see further help.")


toggleBackupProcess()


			